<?php

namespace Drupal\feeds_xsltparser;

/**
 * @file: Apply a map of XSLTPipes keyed by integers or strings
 */
class XSLTMap implements XSLTPipe {
  /**
   * @var \Iterator
   */
  private $map;

  /**
   * @var string
   */
  private $xsltParamKeyName;

  /**
   * @var \Drupal\feeds_xsltparser\XSLTPipe
   */
  private $preproc;

  public function __construct(
    \Iterator $map,
    string $xsltParamKeyName = '_item',
    XSLTPipe $preproc = NULL
  ) {
    $this->map = $map;
    $this->xsltParamKeyName = $xsltParamKeyName;
    $this->preproc = $preproc;
  }

  public function setXsltParamKeyName(string $xsltParamKeyName) {
    $this->xsltParamKeyName = $xsltParamKeyName;
  }

  public function setPreprocessor(XSLTPipe $preproc) {
    $this->preproc = $preproc;
  }

  /**
   * {@inheritdoc}
   */
  public function run(string $markup, $params = []) {
    // Run preprocessor pipe if configured.
    if ($this->preproc) {
      $markup = $this->preproc->run($markup, $params);
    }

    // If the mapping function itself is implemented as an XSLTPipe, like e.g.
    // XSLTRepeat, run it here on the input.
    if ($this->map instanceof XSLTPipe) {
      $this->map->run($markup, $params);
    }

    $result = [];
    foreach ($this->map as $key => $pipe) {
      $pipe_params = $params + [$this->xsltParamKeyName => $key];
      $result[$key] = $pipe->run($markup, $pipe_params);
    }

    return $result;
  }

}
