<?php

namespace Drupal\feeds_xsltparser;

/**
 * @file: Implements an index iterator over one XSLTPipe. The number of times
 * the pipe is repeated is determined by applying the counter pattern.
 */
class XSLTRepeat implements \Iterator, XSLTPipe {
  private $idx;
  private $repetitions;

  /**
   * XSLTPipe which will be repeated for each key.
   *
   * @var \Drupal\feeds_xsltparser\XSLTPipe
   */
  private $pipe;

  /**
   * XSLTChain executed from within XSLTMap::run to determine the number times
   * the pipe should be ran.
   *
   * @var \Drupal\feeds_xsltparser\XSLTChain
   */
  private $counter;

  /**
   * Construct an XSLTRepeat instance.
   *
   * @param $counter
   *   An XSLTChain returning an integer denoting the number of
   *   repetitions
   * @param $pipe
   *   The XSLTPipe which should be repeated.
   */
  public function __construct(XSLTChain $counter, XSLTPipe $pipe) {
    $this->counter = $counter;
    $this->pipe = $pipe;
  }

  /**
   * {@inheritdoc}
   */
  public function current(): mixed {
    return $this->pipe;
  }

  /**
   * {@inheritdoc}
   */
  public function key(): mixed {
    return $this->idx;
  }

  /**
   * {@inheritdoc}
   */
  public function next(): void {
    $this->idx++;
  }

  /**
   * {@inheritdoc}
   */
  public function rewind(): void {
    $this->idx = 0;
  }

  /**
   * {@inheritdoc}
   */
  public function valid(): bool {
    return $this->idx < $this->repetitions;
  }

  /**
   * {@inheritdoc}
   */
  public function run($markup, $params = []) {
    $result = trim($this->counter->run($markup, $params));
    if (!preg_match('/^\d+$/', $result)) {
      throw new \Exception('Result of counter stylesheet must be a positive integer.');
    }

    $this->repetitions = intval($result);
    return $result;
  }

}
